﻿// LocationTemperatures.cs by Charles Petzold, September 2009
using System;
using System.IO;
using System.Net;
using System.Windows.Threading;
using System.Xml;
using System.Xml.Serialization;

namespace TemperatureHistory
{
    public class LocationTemperatures
    {
        string station;
        string localFilename;

        protected LocationTemperatures()
        {
        }

        public string Location { set; get; }
        public TemperatureReadingCollection TemperatureReadings { set; get; }

        public static LocationTemperatures LoadData(string station, string localFilename)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(LocationTemperatures));
            LocationTemperatures locationTemperatures;

            try
            {
                StreamReader reader = File.OpenText(localFilename);
                locationTemperatures = serializer.Deserialize(reader) as LocationTemperatures;
                reader.Close();
            }
            catch
            {
                locationTemperatures = new LocationTemperatures();
                locationTemperatures.TemperatureReadings = new TemperatureReadingCollection();
            }

            locationTemperatures.station = station;
            locationTemperatures.localFilename = localFilename;
            locationTemperatures.BeginReadings();

            return locationTemperatures;
        }

        void BeginReadings()
        {
    //        GetData();

            DispatcherTimer tmr = new DispatcherTimer();
            tmr.Tick += OnTimerTick;
            tmr.Interval = TimeSpan.FromMinutes(15);
            tmr.Start();
        }

        void OnTimerTick(object sender, EventArgs args)
        {
            GetData();
        }

        void GetData()
        {
            WebClient webClient = new WebClient();
            webClient.DownloadStringCompleted += OnWebClientDownloadStringCompleted;
            webClient.DownloadStringAsync(new Uri("http://www.weather.gov/data/current_obs/" + station + ".xml"));
        }

        void OnWebClientDownloadStringCompleted(object sender, DownloadStringCompletedEventArgs args)
        {
            if (args.Error != null)
            {
                // Silently ignore error
                return;
            }

            if (!ParseXml(args.Result))
            {
                // Silently ignore error
                return;
            }
        }

        bool ParseXml(string xml)
        {
            XmlReader reader = XmlReader.Create(new StringReader(xml));
            reader.MoveToContent();

            if (reader.Name != "current_observation")
                return false;

            if (!reader.ReadToDescendant("location"))
                return false;

            string location = reader.Name;

            if (!reader.ReadToNextSibling("observation_time_rfc822"))
                return false;

            DateTime dateTime = DateTime.Parse(reader.ReadElementContentAsString());

            // Check if an entry with the same DateTime is already in the collection
            bool alreadyHaveIt = false;

            foreach (TemperatureReading tempReading in TemperatureReadings)
                alreadyHaveIt |= tempReading.DateTime == dateTime;

            if (alreadyHaveIt)
                return true;

            // Get the Fahrenheit and Celsius temperatures
            if (!reader.ReadToNextSibling("temp_f"))
                return false;

            decimal fahrenheit = reader.ReadElementContentAsDecimal();

            if (!reader.ReadToNextSibling("temp_c"))
                return false;

            decimal celsius = reader.ReadElementContentAsDecimal();

            // This really only has to be done once, but...
            Location = location;

            // Create a new TemperatureReading object
            TemperatureReading temperatureReading = new TemperatureReading()
            {
                DateTime = dateTime,
                FahrenheitTemperature = fahrenheit,
                CelsiusTemperature = celsius
            };

            // Add it to the collection
            TemperatureReadings.Add(temperatureReading);

            // Update the XML file on disk
            XmlSerializer serializer = new XmlSerializer(typeof(LocationTemperatures));
            StreamWriter writer = File.CreateText(localFilename);
            serializer.Serialize(writer, this);
            writer.Close();
            return true;
        }

    }
}